unless Object.const_defined?('Freshen')
  raise "You can't load this file. Please load the freshen file instead."
end

unless Freshen.cli?
  raise "Freshen::CLI should only be used by the command line interface binary"
end

class Freshen
  # The command line interface class.
  #
  class CLI < Thor
    desc "version", "Print the current version number"
    long_desc <<-EOF
      Usage:
      \x5
      \x5 $ freshen version
    EOF
    def version
      puts Freshen::VERSION
    end
    
    desc "list", "List all installed fresheners"
    long_desc <<-EOF
      Usage:
      \x5
      \x5 $ freshen list
    EOF
    def list
      fresheners = Freshener.all.keys
      
      if fresheners.empty?
        puts "No fresheners are installed"
      else
        puts fresheners.join(', ')
      end
    end
    
    desc "info FRESHENER", "Get info about a specific freshener"
    long_desc <<-EOF
      Usage:
      \x5
      \x5 $ freshen info rubygems
    EOF
    def info(freshener)
      begin
        Freshener.ensure_installed! freshener
      rescue Freshen::FreshenerNotInstalled => e
        puts "#{String.ballot.red} #{e}"
        exit! 1
      end
      
      freshener = Freshener.instance_of(freshener)
      
      puts freshener.class.name
      puts "  #{freshener.desc}"
    end
    
    desc "up [FRESHENERS]", "Run all/specified fresheners"
    long_desc <<-EOF
      Example:
      \x5
      \x5 Freshen up everything:
      \x5 $ freshen up 
      \x5
      \x5 Freshen up just rubygems:
      \x5 $ freshen up rubygems
    EOF
    def up(*fresheners)
      stats = {
        successful: {
          color: :green,
          count: 0
        },
        skipped: {
          color: :yellow,
          count: 0
        }
      }
      
      printed_output = false
      
      begin
        duration = Time.duration_of do
          Freshenary.freshen_up(*fresheners) do |name, freshener, status|
            case status
            when Freshenary::Status::FRESHENING
              print "\n" if printed_output
              print String.prefix.green
              puts " Freshening #{freshener.class.name}".bold
              printed_output = true
            when Freshenary::Status::CLEANING_UP
              print "\n" if printed_output
              print String.prefix.green
              puts " Cleaning Up #{freshener.class.name}".bold
              printed_output = true
            when Freshenary::Status::FRESHENED
              stats[:successful][:count] += 1
            when Freshenary::Status::SKIPPED
              print "\n" if printed_output
              print String.prefix.yellow
              puts " Skipped #{freshener.class.name}".bold
              printed_output = true
              
              stats[:skipped][:count] += 1
            end
          end
        end
        
        stats.delete_if { |key, value| 0 == value[:count] }
        
        unless stats.empty?
          stats = stats.map { |key, value| "#{value[:count]} #{key}".send(value[:color]) }
          print "\n" if printed_output
          puts "#{stats.join(" / ")} (%.2f sec)" % duration
        end
      rescue Freshen::FreshenerNotInstalled, Freshen::ExecutableError => e
        print "\n" if printed_output
        puts "#{String.ballot.red} #{e}"
        exit! 1
      end
    end
    
    default_task :up
  end
end
