unless Object.const_defined?('Freshen')
  raise "You can't load this file. Please load the freshen file instead."
end

class Freshen
  # The system class responsible for checking if
  # the current system's kernel is supported.
  #
  class System
    # An array of supported kernels
    #
    attr_reader :supported_kernels
    
    # Setup an instance of this class with an array of
    # supported kernels
    #
    # Example:
    #   >> Freshen::System.new
    #   => #<Freshen:0x00000000000000>
    #
    def initialize
      @supported_kernels = [:darwin, :linux]
    end
    
    # Get the current version of this system's kernel
    #
    # Example (OS X):
    #   >> system = Freshen::System.new
    #   >> system.kernel
    #   => darwin
    #
    # Example (Linux):
    #   >> system = Freshen::System.new
    #   >> system.kernel
    #   => linux
    #
    def kernel
      Gem::Platform.local.os
    end
    
    # A boolean value of whether or not this system's
    # kernel is supported
    #
    # Example:
    #   >> system = Freshen::System.new
    #   >> system.supported?
    #   => true
    #
    def supported?
      @supported_kernels.each do |k|
        return true if k.to_s == kernel
      end
    
      return false
    end
    
    # Check if the current kernel is supported and if not,
    # raise an UnsupportedKernel error
    #
    # Example:
    #   >> system = Freshen::System.new
    #   >> system.check_supported!
    #   => nil
    #
    def check_supported!
      unless supported?
        raise UnsupportedKernel.new(current)
      end
    end
    
    # Check if the current Operating System is supported
    # and if not, raise an UnsupportedOperatingSystem error
    #
    # Example:
    #   >> Freshen::System.check_supported!
    #   => nil
    #
    def self.check_supported!
      system = Freshen::System.new
      system.check_supported!
    end
  end
end

Freshen::System.check_supported!
