unless Object.const_defined?('Freshen')
  raise "You can't load this file. Please load the freshen file instead."
end

class Freshener
  include Freshen::Executable
  
	# A short description of the freshener
	attr_class :desc
  
  # Returns a Hash of all installed fresheners.
  #
  # Example:
  #   >> Freshener.all
  #   => { "rubygems" => #<RubyGems:0x00000000000000> }
  #
  def self.all
    fresheners = Dir.glob(File.join(Freshen::FRESHENERS_DIR, "*.rb"))
    fresheners.map! { |path| File.basename(path, ".rb") }
    instances_of(*fresheners)
  end
  
  # Returns a Hash containing instances of the
  # specified freshehers.
  #
  # Example:
  #   >> Freshener.instances_of :ruby_gems
  #   => { "rubygems" => #<RubyGems:0x00000000000000> }
  #
  def self.instances_of(*fresheners)
    fresheners.sort_by! do |name|
      name
    end
    
    fresheners.map! do |name|
      [name.to_s, instance_of(name.to_s)]
    end
    
    Hash[*fresheners.flatten]
  end
  
  # Returns an instance of the specified freshener.
  #
  # Arguments:
  #   name: (String)
  #
  # Example:
  #   >> Freshener.instance_of :ruby_gems
  #   => #<RubyGems:0x00000000000000>
  #
  def self.instance_of(name)
    load_by_name(name)
    
    klass = class_for(name)
    klass = Object.const_get(klass)
    klass.new
  end
  
  # Get the class name for a freshener.
  #
  # Arguments:
  #   name: (String)
  #
  # Example:
  #   >> Freshener.class_for :ruby_gems
  #   => "RubyGems"
  #
  def self.class_for(name)
    klass = name.to_s.capitalize
    klass.gsub!(/[-_.\s]([a-zA-Z0-9])/) { $1.upcase }
    klass.tr!("+", "x")
    klass
  end
  
  # Load a freshener from the fresheners directory.
  #
  # Arguments:
  #   name: (String)
  #
  # Example:
  #   >> Freshener.load_by_name :ruby_gems
  #   => nil
  #
  # Raises:
  #   FreshenerNotInstalled: When the specified freshener is not installed.
  #   FreshenerLoadError: When the specified freshener could not be loaded.
  #
  def self.load_by_name(name)
    begin
      require File.join(Freshen::FRESHENERS_DIR, "#{name}.rb")
    rescue LoadError
      raise Freshen::FreshenerLoadError.new(name)
    end
    
    nil
  end
  
  # Check that the specified fresheners are installed.
  #
  # Arguments:
  #   fresheners: (Splat)
  #
  # Example:
  #   >> Freshener.installed? :ruby_gems
  #   => true
  #
  def self.installed?(*fresheners)
    fresheners.each do |name|
      return false unless File.file? File.join(Freshen::FRESHENERS_DIR, "#{name}.rb")
    end
    
    true
  end
  
  # Ensure that the specified fresheners are installed.
  #
  # Arguments:
  #   >> Freshener.ensure_installed! :ruby_gems
  #   => nil
  #
  def self.ensure_installed!(*fresheners)
    fresheners.each do |name|
      unless installed? name
        raise Freshen::FreshenerNotInstalled.new(name)
      end
    end
    
    nil
  end
  
  # Mark certain class methods as private
	private_class_method :load_by_name
  
  # Whether or not we the freshener needs
  # freshening up.
  #
  # Example:
  #   >> rubygems = RubyGems.new
  #   >> rubygems.needs_freshening?
  #   => true
  #
  def needs_freshening?
    true
  end
  
	# Freshen up the freshener!
  #
  # Example:
  #   >> rubygems = RubyGems.new
  #   >> rubygems.freshen
  #   => nil
	#
	def freshen
    raise Freshen::UpdateMethodNotImplementedError.new(self)
	end
end